package ru.myitschool.work.data.repo

import dagger.Lazy
import dagger.Reusable
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import ru.myitschool.work.data.mapper.EntryInfoMapper
import ru.myitschool.work.data.mapper.UserInfoMapper
import ru.myitschool.work.data.source.AccountNetworkDataSource
import ru.myitschool.work.data.source.EntryNetworkDataSource
import ru.myitschool.work.domain.entry.entities.EntryEntity
import ru.myitschool.work.domain.entry.repo.EntryRepository
import ru.myitschool.work.domain.profile.entities.UserInfoEntity
import ru.myitschool.work.domain.profile.repo.UserInfoRepository
import ru.myitschool.work.ui.entrylist.adapter.EntryHistoryEntity
import javax.inject.Inject

@Reusable
class EntryRepositoryImpl @Inject constructor(
    private val entryNetworkDataSource: EntryNetworkDataSource,
    private val entryInfoMapper: Lazy<EntryInfoMapper>,
): EntryRepository {
    override suspend fun getEntries(basicAuth: String): Result<List<EntryHistoryEntity>> {
        return withContext(Dispatchers.IO) {
            entryNetworkDataSource.getEntries(basicAuth).fold(
                onSuccess = { value -> entryInfoMapper.get().invoke(value) },
                onFailure = { error -> Result.failure(error) }
            )
        }
    }
}