package ru.myitschool.work.ui.entrylist

import android.content.res.ColorStateList
import androidx.fragment.app.viewModels
import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import androidx.core.content.ContextCompat
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout
import com.squareup.picasso.Picasso
import dagger.hilt.android.AndroidEntryPoint
import ru.myitschool.work.R
import ru.myitschool.work.databinding.FragmentEntryListBinding
import ru.myitschool.work.databinding.FragmentLoginBinding
import ru.myitschool.work.ui.entrylist.adapter.AdapterEntryHistory
import ru.myitschool.work.ui.entrylist.adapter.EntryHistoryEntity
import ru.myitschool.work.ui.login.LoginDestination
import ru.myitschool.work.ui.login.LoginViewModel
import ru.myitschool.work.ui.profile.ProfileDestination
import ru.myitschool.work.ui.profile.ProfileViewModel
import ru.myitschool.work.utils.collectWhenStarted
import ru.myitschool.work.utils.visibleOrGone

@AndroidEntryPoint
class EntryListFragment : Fragment(R.layout.fragment_entry_list) {

    private var _binding: FragmentEntryListBinding? = null
    private val binding: FragmentEntryListBinding get() = _binding!!

    private val viewModel: EntryListViewModel by viewModels()

    private lateinit var swipeRefreshLayout: SwipeRefreshLayout

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentEntryListBinding.bind(view)
        initCallback()
        subscribe()

        swipeRefreshLayout = binding.swipeRefreshLayout2

        swipeRefreshLayout.setOnRefreshListener {
            viewModel.updateEntryList()
        }
    }

    private fun initCallback() {
        binding.floatingActionButton2.setOnClickListener { viewModel.closeEntryList() }


//        binding.recyclerView.adapter = AdapterEntryHistory(viewModel.getEntryList())
        binding.recyclerView.layoutManager = LinearLayoutManager(requireContext())
    }

    private fun subscribe() {
        viewModel.state.collectWhenStarted(this) { state ->
            binding.recyclerView.visibleOrGone(state is EntryListViewModel.State.Show)
            binding.error2.visibleOrGone(state is EntryListViewModel.State.Error)

            when(state) {
                is EntryListViewModel.State.Loading -> {
                    swipeRefreshLayout.isRefreshing = true
                }
                is EntryListViewModel.State.Error -> {
                    swipeRefreshLayout.isRefreshing = false
                    binding.error2.text = state.errorText
                }
                is EntryListViewModel.State.Show -> {
                    swipeRefreshLayout.isRefreshing = false
                    binding.recyclerView.adapter = AdapterEntryHistory(state.entryList)
                }
            }
        }

        viewModel.action.collectWhenStarted(this) { action ->
                when (action) {
                    is EntryListViewModel.Action.OpenProfile -> {
                        findNavController().navigate(ProfileDestination) {
                            popUpTo<ProfileDestination> { inclusive = true }
                        }
                    }
                }
            }
        }
    }