package ru.myitschool.work.ui.entrylist

import android.content.Context
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.Lazy
import dagger.hilt.android.lifecycle.HiltViewModel
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asSharedFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import ru.myitschool.work.R
import ru.myitschool.work.domain.entry.EntryListUseCase
import ru.myitschool.work.ui.entrylist.adapter.EntryHistoryEntity
import ru.myitschool.work.utils.MutablePublishFlow
import javax.inject.Inject


@HiltViewModel
class EntryListViewModel @Inject constructor(
    @ApplicationContext private val context: Context,
    private val entryUseCase: Lazy<EntryListUseCase>,
) : ViewModel() {
    private val _action = MutablePublishFlow<Action>()
    val action = _action.asSharedFlow()

    private val _state = MutableStateFlow<EntryListViewModel.State>(initialState)
    val state = _state.asStateFlow()

    init {
        updateEntryList()
    }

    // Новый список данных
    private var entryList = listOf(
        EntryHistoryEntity("type1", "2023-10-01T12:00:00", "id001"),
        EntryHistoryEntity("type2", "2023-10-02T13:00:00", "id002"),
        EntryHistoryEntity("type3", "2023-10-03T14:00:00", "id003"),
        EntryHistoryEntity("type4", "2023-10-04T15:00:00", "id004"),
        EntryHistoryEntity("type5", "2023-10-05T16:00:00", "id005")
    )

    // Функция для получения данных
    fun updateEntryList() {
        viewModelScope.launch {
            _state.update { State.Loading }
            entryUseCase.get().invoke().fold(
                onSuccess = { entryListEntity ->
                    entryList = entryListEntity
                    _state.update {
                        State.Show(
                            entryList = entryListEntity
                        )
                    }
                },
                onFailure = { error ->
                    _state.update {
                        State.Error(
                            errorText = error.localizedMessage
                                ?: context.resources.getString(R.string.login_error)
                        )
                    }
                }
            )
        }
    }

    fun closeEntryList(){
        viewModelScope.launch {
            _action.emit(Action.OpenProfile)
        }
    }

    sealed interface Action {
        data object OpenProfile: Action
    }

    sealed interface State {
        data object Loading : State

        data class Error(
            val errorText: String,
        ) : State

        data class Show(
            val entryList: List<EntryHistoryEntity>
        ) : State
    }

    private companion object {
        val initialState = State.Loading
    }
}