package ru.myitschool.work.ui.searchuser

import android.content.Context
import android.util.Log
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asSharedFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import ru.myitschool.work.R
import ru.myitschool.work.utils.MutablePublishFlow
import javax.inject.Inject

@HiltViewModel
class SearchUserViewModel @Inject constructor(
    @ApplicationContext private val context: Context,
) : ViewModel() {
    private val _action = MutablePublishFlow<Action>()
    val action = _action.asSharedFlow()

    private val _state = MutableStateFlow<State>(
        initialState
    )
    val state = _state.asStateFlow()

    fun closeSearchUser() {
        viewModelScope.launch {
            _action.emit(Action.OpenProfile)
        }
    }

    fun setLoadingState() {
        viewModelScope.launch {
            _state.update { State.Loading }
        }
    }
    fun openInfo(userName: String){
        viewModelScope.launch {
            _action.emit(Action.OpenInfo(userName))
        }
    }

    fun searchUser(userName: String) {
        viewModelScope.launch {
            setLoadingState()
            // Simulating delay for data retrieval
            kotlinx.coroutines.delay(1000) // Delay for 1 second

            // Mock user check
            val existingUsers = listOf("user1", "user2", "user3") // Mock users
            val userExists = existingUsers.contains(userName)

            if (userExists) {
                openInfo(userName)
            } else {
                // If user does not exist, update state with error
                val errorText = context.resources.getString(R.string.login_error)
                _state.update {
                    State.Error(
                        errorText = Throwable("User not found: $errorText")
                    )
                }
            }
        }
    }

    sealed interface Action {
        data object OpenProfile : Action
        data class OpenInfo(val userName: String): Action
    }

    sealed interface State {
        data object Loading : State

        data class Error(
            val errorText: Throwable,
        ) : State
    }

    companion object {
        val initialState = State.Loading
    }
}